@@ -1,7 +1,8 @@
 # File KGE.R
-# Part of the hydroGOF R package, http://www.rforge.net/hydroGOF/ ; 
-#                                 http://cran.r-project.org/web/packages/hydroGOF/
-# Copyright 2011-2014 Mauricio Zambrano-Bigiarini
+# Part of the hydroGOF R package, https://github.com/hzambran/hydroGOF
+#                                 https://cran.r-project.org/package=hydroGOF
+#                                 http://www.rforge.net/hydroGOF/ ;
+# Copyright 2011-2023 Mauricio Zambrano-Bigiarini
 # Distributed under GPL 2 or later
 
 ################################################################################
@@ -15,6 +16,8 @@
 #          18-Oct-2012 ; 19-Oct-2012                                           #
 #          24-Jan-2014                                                         #
 #          28-Feb-2016 ; 17-Jul-2016                                           #
+#          12-Jul-2022 ; 13-Jul-2022                                           #
+#          11-Jul-2023 ; 08-Nov-2023                                           #
 ################################################################################
 # The optimal value of KGE is 1
 
@@ -31,6 +34,11 @@
 # Journal of Hydrology, Volumes 424-425, 6 March 2012, Pages 264-277, 
 # DOI:10.1016/j.jhydrol.2012.01.011.
 
+# Ref3: Tang, G., Clark, M. P., & Papalexiou, S. M. (2021).  
+# SC-earth: a station-based serially complete earth dataset from 1950 to 2019. 
+# Journal of Climate, 34(16), 6493-6511.
+# DOI: 10.1175/JCLI-D-21-0067.1.
+
 
 # 'obs' : numeric 'data.frame', 'matrix' or 'vector' with observed values
 # 'sim' : numeric 'data.frame', 'matrix' or 'vector' with simulated values
@@ -41,7 +49,10 @@
 KGE <- function(sim, obs, ...) UseMethod("KGE")
 
 KGE.default <- function(sim, obs, s=c(1,1,1), na.rm=TRUE, 
-                        method=c("2009", "2012"), out.type=c("single", "full"), ...) { 
+                        method=c("2009", "2012", "2021"), out.type=c("single", "full"), 
+                        fun=NULL, ...,
+                        epsilon.type=c("none", "Pushpalatha2012", "otherFactor", "otherValue"), 
+                        epsilon.value=NA) { 
 
   # If the user provided a value for 's'
   if (!identical(s, c(1,1,1)) )  {
@@ -63,6 +74,14 @@ KGE.default <- function(sim, obs, s=c(1,1,1), na.rm=TRUE,
     obs <- as.numeric(obs[vi])
     sim <- as.numeric(sim[vi])
 
+    if (!is.null(fun)) {
+      fun1 <- match.fun(fun)
+      new  <- preproc(sim=sim, obs=obs, fun=fun1, ..., 
+                      epsilon.type=epsilon.type, epsilon.value=epsilon.value)
+      sim  <- new[["sim"]]
+      obs  <- new[["obs"]]
+    } # IF end
+
     # Mean values
     mean.sim <- mean(sim, na.rm=na.rm)
     mean.obs <- mean(obs, na.rm=na.rm)
@@ -79,6 +98,10 @@ KGE.default <- function(sim, obs, s=c(1,1,1), na.rm=TRUE,
 
     # Beta is the ratio between the mean of the simulated values to the mean of observations
     Beta <- mean.sim / mean.obs
+
+    # Beta.2021 is the bias term proposed by Tang et al. (2021) to avoid the 
+    # anomalously negative KE or KGE' values when the mean value is close to zero 
+    Beta.2021 <- (mean.sim - mean.obs) / sigma.obs
        
     # CV.sim is the coefficient of variation of the simulated values [dimensionless]
     # CV.obs is the coefficient of variation of the observations [dimensionless]
@@ -89,17 +112,28 @@ KGE.default <- function(sim, obs, s=c(1,1,1), na.rm=TRUE,
     Gamma <- CV.sim / CV.obs
        
     # Variability ratio depending on 'method'
-    if(method=="2012") {
+    if (method=="2012") {
+      br     <- Beta
+      br.stg <- "Beta"
       vr     <- Gamma
       vr.stg <- "Gamma"
-    } else {
+    } else if (method=="2009") {
+        br     <- Beta
+        br.stg <- "Beta"
         vr     <- Alpha
         vr.stg <- "Alpha"
-      } # ELSE end
+      } else if (method=="2021") {
+          br     <- Beta.2021
+          br.stg <- "Beta.2021"
+          vr     <- Alpha
+          vr.stg <- "Alpha"
+        } # ELSE end
 
     # KGE Computation
     if ( (mean.obs != 0) | (sigma.obs != 0) ) {
-        KGE <- 1 - sqrt( (s[1]*(r-1))^2 + (s[2]*(vr-1))^2 + (s[3]*(Beta-1))^2 )
+        if ( (method=="2009") | (method=="2012") ) {
+          KGE <- 1 - sqrt( (s[1]*(r-1))^2 + (s[2]*(vr-1))^2 + (s[3]*(Beta-1))^2 )
+        } else KGE <- 1 - sqrt( (s[1]*(r-1))^2 + (s[2]*(vr-1))^2 + (s[3]*(Beta.2021))^2 )
     } else {
         if ( mean.obs != 0)  warning("Warning: 'mean(obs)==0'. Beta = Inf")
         if ( sigma.obs != 0) warning("Warning: 'sd(obs)==0'. ", vr.stg, " = Inf")
@@ -110,9 +144,17 @@ KGE.default <- function(sim, obs, s=c(1,1,1), na.rm=TRUE,
       r    <- NA
       Beta <- NA
       vr   <- NA
-      if(method=="2012") {
+      br   <- NA
+      if (method=="2012") {
+        br.stg <- "Beta"
         vr.stg <- "Gamma"
-      } else vr.stg <- "Alpha" 
+      } else if (method=="2009") {
+          br.stg <- "Beta"
+          vr.stg <- "Alpha" 
+        } else {
+            br.stg <- "Beta.2021"
+            vr.stg <- "Alpha" 
+          } # ELSE end
       KGE <- NA
       warning("There are no pairs of 'sim' and 'obs' without missing values !")
     } # ELSE end
@@ -120,8 +162,8 @@ KGE.default <- function(sim, obs, s=c(1,1,1), na.rm=TRUE,
   if (out.type=="single") {
         out <- KGE
   } else {
-      out <- list(KGE.value=KGE, KGE.elements=c(r, Beta, vr))
-      names(out[[2]]) <- c("r", "Beta", vr.stg)
+      out <- list(KGE.value=KGE, KGE.elements=c(r, br, vr))
+      names(out[[2]]) <- c("r", br.stg, vr.stg)
     } # ELSE end    
  
   return(out)
@@ -136,9 +178,14 @@ KGE.default <- function(sim, obs, s=c(1,1,1), na.rm=TRUE,
 # Updates: 25-Aug-2011                                                         #
 #          10-Oct-2012                                                         #
 #          18-Oct-2012 ; 19-Oct-2012                                           #
+#          12-Jul-2022 ; 13-Jul-2022                                           #
+#          11-Jul-2023                                                         #
 ################################################################################
 KGE.matrix <- function (sim, obs, s=c(1,1,1), na.rm=TRUE, 
-                        method=c("2009", "2012"), out.type=c("single", "full"), ...){ 
+                        method=c("2009", "2012", "2021"), out.type=c("single", "full"), 
+                        fun=NULL, ...,
+                        epsilon.type=c("none", "Pushpalatha2012", "otherFactor", "otherValue"), 
+                        epsilon.value=NA) { 
 
   # Checking that 'sim' and 'obs' have the same dimensions
   if ( all.equal(dim(sim), dim(obs)) != TRUE )
@@ -164,11 +211,17 @@ KGE.matrix <- function (sim, obs, s=c(1,1,1), na.rm=TRUE,
           
   if (out.type=="single") {
     out <- sapply(1:ncol(obs), function(i,x,y) { 
-                   KGE[i] <- KGE.default( x[,i], y[,i], s=s, na.rm=na.rm, method=method, out.type=out.type, ... )
+                   KGE[i] <- KGE.default( x[,i], y[,i], s=s, na.rm=na.rm, 
+                                         method=method, out.type=out.type, 
+                                         fun=fun, ..., epsilon.type=epsilon.type, 
+                                         epsilon.value=epsilon.value )
                  }, x=sim, y=obs )  
     names(out) <- colnames(obs) 
   } else { out <- lapply(1:ncol(obs), function(i,x,y) { 
-                         KGE.default( x[,i], y[,i], s=s, na.rm=na.rm, method=method, out.type=out.type, ... )
+                         KGE.default( x[,i], y[,i], s=s, na.rm=na.rm, method=method, 
+                                      out.type=out.type, fun=fun, ..., 
+                                      epsilon.type=epsilon.type, 
+                                      epsilon.value=epsilon.value )
                        }, x=sim, y=obs ) 
             for (i in 1:length(out) ) {
                KGE[i] <- out[[i]][[1]]
@@ -189,9 +242,14 @@ KGE.matrix <- function (sim, obs, s=c(1,1,1), na.rm=TRUE,
 # Updates: 25-Aug-2011                                                         #
 #          10-Oct-2012                                                         #
 #          18-Oct-2012 ; 19-Oct-2012                                           #
+#          12-Jul-2022 ; 13-Jul-2022                                           #
+#          11-Jul-2023                                                         #
 ################################################################################
 KGE.data.frame <- function (sim, obs, s=c(1,1,1), na.rm=TRUE, 
-                            method=c("2009", "2012"), out.type=c("single", "full"), ...){ 
+                            method=c("2009", "2012", "2021"), out.type=c("single", "full"), 
+                            fun=NULL, ...,
+                            epsilon.type=c("none", "Pushpalatha2012", "otherFactor", "otherValue"), 
+                            epsilon.value=NA) { 
  
   sim <- as.matrix(sim)
   obs <- as.matrix(obs)
@@ -199,7 +257,8 @@ KGE.data.frame <- function (sim, obs, s=c(1,1,1), na.rm=TRUE,
   method   <- match.arg(method)
   out.type <- match.arg(out.type) 
    
-  KGE.matrix(sim, obs, s=s, na.rm=na.rm, method=method, out.type=out.type, ...)
+  KGE.matrix(sim, obs, s=s, na.rm=na.rm, method=method, out.type=out.type, 
+             fun=fun, ..., epsilon.type=epsilon.type, epsilon.value=epsilon.value)
      
 } # 'KGE.data.frame' end
 
@@ -209,15 +268,22 @@ KGE.data.frame <- function (sim, obs, s=c(1,1,1), na.rm=TRUE,
 ################################################################################
 # Started: 22-Mar-2013                                                         #
 # Updates: 16-Aug-2016                                                         #
+#          12-Jul-2022 ; 13-Jul-2022                                           #
+#          11-Jul-2023                                                         #
 ################################################################################
 KGE.zoo <- function(sim, obs, s=c(1,1,1), na.rm=TRUE, 
-                    method=c("2009", "2012"), out.type=c("single", "full"), ...){
+                    method=c("2009", "2012", "2021"), out.type=c("single", "full"), 
+                    fun=NULL, ...,
+                    epsilon.type=c("none", "Pushpalatha2012", "otherFactor", "otherValue"), 
+                    epsilon.value=NA) { 
     
     sim <- zoo::coredata(sim)
     if (is.zoo(obs)) obs <- zoo::coredata(obs)
     
     if (is.matrix(sim) | is.data.frame(sim)) {
-       KGE.matrix(sim, obs, s=s, na.rm=na.rm, method=method, out.type=out.type, ...)
-    } else NextMethod(sim, obs, s=s, na.rm=na.rm, method=method, out.type=out.type, ...)
+       KGE.matrix(sim, obs, s=s, na.rm=na.rm, method=method, out.type=out.type, 
+                  fun=fun, ..., epsilon.type=epsilon.type, epsilon.value=epsilon.value)
+    } else NextMethod(sim, obs, s=s, na.rm=na.rm, method=method, out.type=out.type, 
+                      fun=fun, ..., epsilon.type=epsilon.type, epsilon.value=epsilon.value)
      
   } # 'KGE.zoo' end
