@@ -1,7 +1,8 @@
 # File plot2.R
-# Part of the hydroGOF R package, http://www.rforge.net/hydroGOF/ ; 
-#                                 http://cran.r-project.org/web/packages/hydroGOF/
-# Copyright 2011-2017 Mauricio Zambrano-Bigiarini
+# Part of the hydroGOF R package, https://github.com/hzambran/hydroGOF ; 
+#                                 https://cran.r-project.org/package=hydroGOF
+#                                 http://www.rforge.net/hydroGOF/
+# Copyright 2010-2024 Mauricio Zambrano-Bigiarini
 # Distributed under GPL 2 or later
 
 ################################################################################
@@ -18,6 +19,7 @@
 #          23-Jan-2012                                                         #
 #          15-Apr-2013 ; 15-May-2013                                           #
 #          06-Aug-2017                                                         #
+#          28-Dec-2022                                                         #
 ################################################################################
                 
 plot2 <- function (x, y, 
@@ -107,9 +109,9 @@ plot2 <- function (x, y,
   
     # class(time(x))== "Date" for 'daily' and 'monthly' time series
     # class(time(x))== "character" for 'annual' time series
-    if ( class(time(x)) == "Date" ) {
+    if ( inherits(time(x), "Date" ) ) {
         y <- vector2zoo(y, dates=time(x)) # hydroTSM::vector2zoo
-    } else if ( class(time(x)) == "character" ) {
+    } else if ( inherits(time(x), "character") ) {
         y <- vector2zoo(y, dates=time(x), date.fmt="%Y") # hydroTSM::vector2zoo
         time(x) <- time(y) #'annual' time series
     } # ELSE END
