@@ -1,7 +1,8 @@
 # File plotbands.R
-# Part of the hydroGOF R package, http://www.rforge.net/hydroGOF/ ; 
-#                                 http://cran.r-project.org/web/packages/hydroGOF/
-# Copyright 2009-2017 Mauricio Zambrano-Bigiarini
+# Part of the hydroGOF R package, https://github.com/hzambran/hydroGOF ; 
+#                                 https://cran.r-project.org/package=hydroGOF
+#                                 http://www.rforge.net/hydroGOF/
+# Copyright 2009-2024 Mauricio Zambrano-Bigiarini
 # Distributed under GPL 2 or later
 
 ################################################################################
@@ -13,6 +14,8 @@
 #          15-Apr-2011 ; 17-May-2011                                           #
 #          15-Apr-2013                                                         #
 #          06-Aug-2017                                                         #
+#          28-Dec-2022                                                         #
+#          20-Jan-2024                                                         #
 ################################################################################
 plotbands <- function(x, lband, uband, sim,
                       
@@ -102,8 +105,8 @@ plotbands <- function(x, lband, uband, sim,
       if ( zoo::is.zoo(x) ) {
         # class(time(x))== "Date" for 'daily' and 'monthly' time series
         # class(time(x))== "character" for 'annual' time series
-        if ( class(time(x)) == "Date" ) { dates <- time(x) 
-        } else if ( class(time(x)) == "character" ) {  
+        if ( inherits(time(x), "Date") ) { dates <- time(x) 
+        } else if ( inherits(time(x), "character") ) {  
              dates <- as.Date(time(x), format="%Y") 
           }  
       } else # If there is no way to obtain the dates
@@ -204,7 +207,7 @@ plotbands <- function(x, lband, uband, sim,
 
     # Draws custom ticks and labels on the X axis
     if ( zoo::is.zoo(x) | xts::is.xts(x) ) {
-      drawTimeAxis(x, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt, cex.axis=cex.axis) # hydroTSM::drawTimeAxis
+      hydroTSM::drawTimeAxis(x, tick.tstep=tick.tstep, lab.tstep=lab.tstep, lab.fmt=lab.fmt, cex.axis=cex.axis) # hydroTSM::drawTimeAxis
     } else axis(side = 1, labels = TRUE)
 
     # Plotting the OBSERVED time series, over the polygons
@@ -216,7 +219,7 @@ plotbands <- function(x, lband, uband, sim,
       
     # Plotting the SIMULATED time series, over the polygons
     if ( !missing(sim) ) {
-        if ( (zoo::is.zoo(x)) & (!xts::is.xts(sim)) ) sim <- xts::as.xts(sim)
+        if ( (zoo::is.zoo(x)) & (!is.zoo(sim)) ) sim <- zoo::as.zoo(sim)
         # Plotting the SIMULATED time series, over the polygons
         if (type[2] == "lines") {
           lines(sim, cex= cex[2], col=col[2], lty=lty[2], lwd=lwd[2], pch=pch[2], ... )
